package minecraft.gui.mvp;


import minecraft.player.InvalidCharacterParamsException;
import minecraft.player.Steve;

import javax.swing.*;

public class StevePresenter implements SteveView.SteveViewListener {
    private final SteveView view;
    private final SteveRepository repository;

    public StevePresenter(SteveView view, SteveRepository repository) {
        this.view = view;
        this.repository = repository;
        this.view.setListener(this);
        loadAllSteves();
    }

    private void loadAllSteves() {
        view.displayAllSteves(repository.getAllSteves());
    }

    @Override
    public void onAddSteve() {
        try {
            Steve steve = createSteveFromForm();
            repository.addSteve(steve);
            view.displayAllSteves(repository.getAllSteves());
            view.clearForm();
            view.displaySuccess("Steve added successfully!");
        } catch (InvalidCharacterParamsException e) {
            view.displayError(e.getMessage());
        }
    }

    @Override
    public void onUpdateSteve() {
        int selectedIndex = view.getSelectedIndex();
        if (selectedIndex < 0) {
            view.displayError("Please select a Steve to update");
            return;
        }

        try {
            Steve updatedSteve = createSteveFromForm();
            repository.updateSteve(selectedIndex, updatedSteve);
            view.displayAllSteves(repository.getAllSteves());
            view.displaySuccess("Steve updated successfully!");
        } catch (InvalidCharacterParamsException e) {
            view.displayError(e.getMessage());
        }
    }

    @Override
    public void onDeleteSteve() {
        int selectedIndex = view.getSelectedIndex();
        if (selectedIndex < 0) {
            view.displayError("Please select a Steve to delete");
            return;
        }

        int confirm = JOptionPane.showConfirmDialog(
                (JFrame) view,
                "Are you sure you want to delete this Steve?",
                "Confirm Delete",
                JOptionPane.YES_NO_OPTION
        );

        if (confirm == JOptionPane.YES_OPTION) {
            repository.removeSteve(selectedIndex);
            view.displayAllSteves(repository.getAllSteves());
            view.clearForm();
            view.displaySuccess("Steve deleted successfully!");
        }
    }

    @Override
    public void onClearForm() {
        view.clearForm();
    }

    @Override
    public void onSteveSelected(int index) {
        if (index >= 0 && index < repository.getAllSteves().size()) {
            view.displaySteveDetails(repository.getSteve(index));
        }
    }

    private Steve createSteveFromForm() throws InvalidCharacterParamsException {
        return new Steve(
                view.getName(),
                view.getHealth(),
                view.getMaxHealth(),
                view.getAttackPower(),
                view.getDefensePower(),
                view.getHunger(),
                view.getMaxHunger(),
                view.isGodMode()
        );
    }
}
