package minecraft.gui.mvp;

import minecraft.player.Steve;

import javax.swing.*;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;
import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.List;

public class SwingSteveView extends JFrame implements SteveView {
    private SteveViewListener listener;
    private JList<Steve> steveList;
    private DefaultListModel<Steve> listModel;

    private JTextField nameField = new JTextField();
    private JTextField healthField = new JTextField();
    private JTextField maxHealthField = new JTextField();
    private JTextField attackPowerField = new JTextField();
    private JTextField defensePowerField = new JTextField();
    private JTextField hungerField = new JTextField();
    private JTextField maxHungerField = new JTextField();
    private JCheckBox godModeCheckBox = new JCheckBox("God Mode");

    public SwingSteveView() {
        super("Steve CRUD - MVP Pattern");
        setSize(800, 600);
        setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
        setLayout(new BorderLayout());

        initComponents();
    }

    private void initComponents() {
        JPanel formPanel = new JPanel(new GridLayout(0, 2, 5, 5));
        formPanel.setBorder(BorderFactory.createTitledBorder("Steve Details"));

        formPanel.add(new JLabel("Name:"));
        formPanel.add(nameField);
        formPanel.add(new JLabel("Health:"));
        formPanel.add(healthField);
        formPanel.add(new JLabel("Max Health:"));
        formPanel.add(maxHealthField);
        formPanel.add(new JLabel("Attack Power:"));
        formPanel.add(attackPowerField);
        formPanel.add(new JLabel("Defense Power:"));
        formPanel.add(defensePowerField);
        formPanel.add(new JLabel("Hunger:"));
        formPanel.add(hungerField);
        formPanel.add(new JLabel("Max Hunger:"));
        formPanel.add(maxHungerField);
        formPanel.add(new JLabel(""));
        formPanel.add(godModeCheckBox);

        JPanel listPanel = new JPanel(new BorderLayout());
        listModel = new DefaultListModel<>();
        steveList = new JList<>(listModel);
        steveList.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);

        steveList.addListSelectionListener(new ListSelectionListener() {
            @Override
            public void valueChanged(ListSelectionEvent e) {
                if (!e.getValueIsAdjusting() && listener != null) {
                    listener.onSteveSelected(steveList.getSelectedIndex());
                }
            }
        });

        listPanel.add(new JScrollPane(steveList), BorderLayout.CENTER);

        JPanel buttonPanel = new JPanel(new FlowLayout());

        JButton addButton = new JButton("Add");
        addButton.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent e) {
                if (listener != null) {
                    listener.onAddSteve();
                }
            }
        });

        JButton updateButton = new JButton("Update");
        updateButton.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent e) {
                if (listener != null) {
                    listener.onUpdateSteve();
                }
            }
        });

        JButton deleteButton = new JButton("Delete");
        deleteButton.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent e) {
                if (listener != null) {
                    listener.onDeleteSteve();
                }
            }
        });

        JButton clearButton = new JButton("Clear");
        clearButton.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent e) {
                if (listener != null) {
                    listener.onClearForm();
                }
            }
        });

        buttonPanel.add(addButton);
        buttonPanel.add(updateButton);
        buttonPanel.add(deleteButton);
        buttonPanel.add(clearButton);

        add(formPanel, BorderLayout.NORTH);
        add(listPanel, BorderLayout.CENTER);
        add(buttonPanel, BorderLayout.SOUTH);
    }

    @Override
    public void displayAllSteves(List<Steve> steves) {
        listModel.clear();
        for (Steve steve : steves) {
            listModel.addElement(steve);
        }
    }

    @Override
    public void displayError(String message) {
        JOptionPane.showMessageDialog(this, message, "Error", JOptionPane.ERROR_MESSAGE);
    }

    @Override
    public void displaySuccess(String message) {
        JOptionPane.showMessageDialog(this, message, "Success", JOptionPane.INFORMATION_MESSAGE);
    }

    @Override
    public void clearForm() {
        nameField.setText("");
        healthField.setText("");
        maxHealthField.setText("");
        attackPowerField.setText("");
        defensePowerField.setText("");
        hungerField.setText("");
        maxHungerField.setText("");
        godModeCheckBox.setSelected(false);
    }

    @Override
    public void displaySteveDetails(Steve steve) {
        setName(steve.getName());
        setHealth(steve.getHealth());
        setMaxHealth(steve.getMaxHealth());
        setAttackPower(steve.getAttackPower());
        setDefensePower(steve.getDefensePower());
        setHunger(steve.getHunger());
        setMaxHunger(steve.getMaxHunger());
        setGodMode(steve.isGodMode());
    }

    @Override
    public String getName() {
        return nameField.getText();
    }

    @Override
    public int getHealth() {
        try {
            return Integer.parseInt(healthField.getText());
        } catch (NumberFormatException e) {
            return 0;
        }
    }

    @Override
    public int getMaxHealth() {
        try {
            return Integer.parseInt(maxHealthField.getText());
        } catch (NumberFormatException e) {
            return 0;
        }
    }

    @Override
    public int getAttackPower() {
        try {
            return Integer.parseInt(attackPowerField.getText());
        } catch (NumberFormatException e) {
            return 0;
        }
    }

    @Override
    public int getDefensePower() {
        try {
            return Integer.parseInt(defensePowerField.getText());
        } catch (NumberFormatException e) {
            return 0;
        }
    }

    @Override
    public int getHunger() {
        try {
            return Integer.parseInt(hungerField.getText());
        } catch (NumberFormatException e) {
            return 0;
        }
    }

    @Override
    public int getMaxHunger() {
        try {
            return Integer.parseInt(maxHungerField.getText());
        } catch (NumberFormatException e) {
            return 0;
        }
    }

    @Override
    public boolean isGodMode() {
        return godModeCheckBox.isSelected();
    }

    @Override
    public void setName(String name) {
        nameField.setText(name);
    }

    @Override
    public void setHealth(int health) {
        healthField.setText(String.valueOf(health));
    }

    @Override
    public void setMaxHealth(int maxHealth) {
        maxHealthField.setText(String.valueOf(maxHealth));
    }

    @Override
    public void setAttackPower(int attackPower) {
        attackPowerField.setText(String.valueOf(attackPower));
    }

    @Override
    public void setDefensePower(int defensePower) {
        defensePowerField.setText(String.valueOf(defensePower));
    }

    @Override
    public void setHunger(int hunger) {
        hungerField.setText(String.valueOf(hunger));
    }

    @Override
    public void setMaxHunger(int maxHunger) {
        maxHungerField.setText(String.valueOf(maxHunger));
    }

    @Override
    public void setGodMode(boolean godMode) {
        godModeCheckBox.setSelected(godMode);
    }

    @Override
    public void setSelectedIndex(int index) {
        steveList.setSelectedIndex(index);
    }

    @Override
    public int getSelectedIndex() {
        return steveList.getSelectedIndex();
    }

    @Override
    public void setListener(SteveViewListener listener) {
        this.listener = listener;
    }
}
